package ttt.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import ttt.model.TTTGame;

import java.util.*;

/** Overall GUI to hold the game and view and run the program. */
public class TTTGui implements ActionListener, Observer {
    
    // fields
    private JFrame frame;
    private JMenuItem resetItem, homestarItem, crapItem;
    private TTTView view;
    private TTTGame game;
    
    /** Constructs the Tic-Tac-Toe GUI. */
    public TTTGui() {
        // construct various fields
        game = new TTTGame(new String[] {"Marty", "Jethro"});
        game.addObserver(this);
        
        frame = new JFrame();
        frame.setTitle("Tic - Tac - D'oh!");
        frame.setSize(500, 400);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        
        // menu bar
        createMenuBar();
        
        JToolBar toolbar = new JToolBar();
        GuiWidgetFactory.createButton(toolbar, new ResetAction());
        
        // layout
        Container contentPane = frame.getContentPane();
        contentPane.add(toolbar, "North");
        
        view = new TTTCrapView(game);
        contentPane.add(view);
        
        TTTPlayersPanel players = new TTTPlayersPanel(game);
        JPanel temp = new JPanel();
        
        players.setBorder(BorderFactory.createTitledBorder("CSE Rul3z!1"));
        
        temp.setBackground(Color.RED);
        
        temp.add(players);
        contentPane.add(new JScrollPane(temp), "West");
        
        frame.setVisible(true);
    }
    
    public void actionPerformed(ActionEvent event) {
        Object source = event.getSource();
        
        if (crapItem == source || homestarItem == source) {
            // switch views
            Container contentPane = frame.getContentPane();
            contentPane.remove(view);
            view.detach();
            
            if (crapItem == source) {
                view = new TTTCrapView(game); // switch to crap view
            } else { // homestarItem == source
                view = new TTTHomestarView(game); // switch to Homestar Runner
                                                // view
            }
            
            contentPane.add(view);
            contentPane.validate(); // does layout again (so new view will show
                                    // up)
        }
    }
    
    public void update(Observable o, Object arg) {
        if (game.isGameOver()) {
            int choice = JOptionPane.showConfirmDialog(frame,
                    "Game is over!  Winner was Player " + game.getWinner()
                            + "\nPlay again?", "Game over",
                    JOptionPane.YES_NO_OPTION);
            
            if (choice == JOptionPane.YES_OPTION) {
                // they want to play again
                game.newGame();
                
                // update the button text
            }
        }
    }
    
    private void createMenuBar() {
        JMenuBar bar = new JMenuBar();
        
        // Factory pattern (usage)
        JMenu gameMenu = GuiWidgetFactory.createMenu("Game", bar);
        resetItem = GuiWidgetFactory
                .createMenuItem(gameMenu, new ResetAction());
        resetItem.setIcon(new ImageIcon("reset.jpg"));
        
        GuiWidgetFactory.createMenuItem(gameMenu, new UndoAction());
        
        JMenu viewMenu = GuiWidgetFactory.createMenu("View", bar);
        
        homestarItem = GuiWidgetFactory.createMenuItemWithHotkey(
                "Homestar Runner", viewMenu, this);
        
        crapItem = GuiWidgetFactory.createMenuItem("Crappy", viewMenu, this);
        
        frame.setJMenuBar(bar);
    }
    
    // Command pattern (implementation)
    private class UndoAction extends AbstractAction {
        private static final long serialVersionUID = 0;

        public UndoAction() {
            super("Undo");
            putValue(Action.MNEMONIC_KEY, new Integer('U'));
            putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke('Z',
                    KeyEvent.CTRL_MASK));
        }
        
        public void actionPerformed(ActionEvent e) {
            game.undoLastMove();
        }
    }
    
    // Command pattern (implementation)
    private class ResetAction extends AbstractAction {
        private static final long serialVersionUID = 0;

        public ResetAction() {
            super("Reset", new ImageIcon("reset.jpg"));
            putValue(Action.MNEMONIC_KEY, new Integer('R'));
            putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke('R',
                    KeyEvent.CTRL_MASK));
        }
        
        public void actionPerformed(ActionEvent e) {
            game.reset();
        }
    }
}
